



CREATE PROCEDURE SQLDriveSpaceWarningEmail
    @WarningThreshold   decimal(4,2) = '90',
    @AlertEmail varchar(200) = '<enter your default email address here>'
AS

-- =============================================
-- Author: Shaun Stuart, shaunjstuart.com
-- Create date: August 24, 2010
-- Description: Sends email when disk space usage is low.
--		To be used with disk space monitoring procedure
--		found at http://shaunjstuart.com/archive/2011/07/drive-space-monitoring-page/
-- =============================================

 
DECLARE @AlertSubject varchar(200)
DECLARE @AlertBody varchar(3000)
 
DECLARE @Server_name varchar(100)
DECLARE @Drive_letter char(2)
DECLARE @SQLPercentFull decimal(4,2)
DECLARE @TotalPercentFull decimal(4,2)
 
SET @AlertSubject = 'ALERT!!! SQL DRIVE SPACE WARNING ON SERVER '
 
DECLARE OverThresholdCursor CURSOR
FOR
    SELECT  server_name,
            Drive_letter,
            (SUM(CAST(size_in_mb as decimal (9,2)))/MAX(CAST(maxdrivesize_in_mb as decimal(9,2)))) * 100 as SQLPercentFull,
            ((MAX(CAST(maxdrivesize_in_mb as decimal(9,2)))- MAX(CAST(drivefreespace_in_mb as decimal(9,2))))/MAX(CAST(maxdrivesize_in_mb as decimal(9,2)))) * 100 as TotalPercentFull
    FROM DatabaseDiskStats
    WHERE statDate = CONVERT([date],GETDATE(),(0))
        AND MaxDriveSize_in_MB <>0
    GROUP BY    Server_name,
                Drive_Letter,
                StatDate
    HAVING  (((MAX(CAST(maxdrivesize_in_mb as decimal(9,2)))- MAX(CAST(drivefreespace_in_mb as decimal(9,2))))/MAX(CAST(maxdrivesize_in_mb as decimal(9,2)))) * 100) >= @WarningThreshold
    ORDER BY server_name,drive_letter
 
OPEN OverThresholdCursor
 
FETCH NEXT FROM OverThresholdCursor INTO @Server_name, @Drive_letter, @SQLPercentFull, @TotalPercentFull
 
WHILE @@fetch_status = 0
BEGIN
    SET @AlertSubject = @AlertSubject + UPPER(@Server_name) + '!!!'
    SET @AlertBody = 'Drive ' + @Drive_letter + ' on SQL Server ' + @Server_name + ' is ' + cast(@TotalPercentFull as varchar(5))
                    + '% full. SQL Server files are using ' + cast(@SQLPercentFull as varchar(5)) + '% of the disk.'
    exec msdb..sp_send_dbmail   @recipients = @AlertEmail,
                @body  = @AlertBody,
                @subject = @AlertSubject
 
    FETCH NEXT FROM OverThresholdCursor INTO @Server_name, @Drive_letter, @SQLPercentFull, @TotalPercentFull
 
END
 
CLOSE   OverThresholdCursor
DEALLOCATE  OverThresholdCursor

Read more: http://shaunjstuart.com/archive/2010/11/drive-space-monitoring-gets-an-update-update/#ixzz1TE43D9RE
Under Creative Commons License: Attribution Non-Commercial No Derivatives
